/**
 * \file
 * \author Matthias Riedl
 * \brief covers the functionality of a limits_alarm FB
 * \improved by usage of methods for data exchange
 * \verbatim
 * $id
 * \endverbatim
 */

#ifndef __LIMITS_ALARM_H
#define __LIMITS_ALARM_H

//com

//std

// own
#include "fb.h"
#include "hysteresis.h"

/**
 * Function Block interface for checking the process value exceeding limits.
 */
class limits_alarm: public fb
{
public:
  /**
   * Sets the input values of the function block. 
   * Should be called before \ref en().
   * \param x - value of x (process value)
   * \param h - value of high limit
   * \param l - value of low limit
   * \param eps - value of eps 
   */ 
  virtual void  set_input(const float x, const float h, const float l, const float eps) = 0;
  /**
   * Gets the input values of the function block. 
   * Should be called before \ref en().
   * \param x - value of x (process value)
   * \param h - value of high limit
   * \param l - value of low limit
   * \param eps - value of eps 
   */ 
  virtual void  get_input(float& x, float& h, float& l, float& eps) const = 0;
  /**
   * Delivers the result of the algorithm.
   * Should be called after \ref en().
   * return true - value exceeds the limit. 
   */
  virtual bool  get_result() const = 0;	
  /**
   * Is true, if value exceeds the high limit.
   * return true - value exceeds the high limit.  
   */
  virtual bool  is_high_alert() const = 0;	
  /**
   * Is true, if value exceeds the low limit.
   * return true - value exceeds the low limit.  
   */
  virtual bool  is_low_alert() const = 0;	
  /**
   * Sets the interface pointer to an instance for lower checker.
   * \param hyst - interface pointer to a hysteresis implementation
   */ 
  virtual void  set_lower_checker(hysteresis* hyst) = 0;
  /**
   * Sets the interface pointer to an instance for upper checker.
   * \param hyst - interface pointer to a hysteresis implementation
   */ 
  virtual void  set_upper_checker(hysteresis* hyst) = 0;
}; // limits_alarm

#endif  //__LIMITS_ALARM_H
