/**
 * \file
 * \author Matthias Riedl
 * \brief covers the functionality of a limits_alarm FB
 * \improved by usage of abstract interfaces
 * \verbatim
 * $id
 * \endverbatim
 */

#ifndef __LIMITS_ALARM_IMPL_H
#define __LIMITS_ALARM_IMPL_H

//com

//std

// own
#include "fb.h"
#include "limits_alarm.h"
#include "hysteresis.h"

/**
 * Function Block implementation for checking the process value exceeding limits.
 */
class limits_alarm_impl: public limits_alarm
{
  // no copy or assignment operations
  limits_alarm_impl(const limits_alarm_impl& obj);
  limits_alarm_impl& operator=(const limits_alarm_impl& obj);
public:
	/** \name Construct/Destructor */
	//! \{
  	/**
   	 * Constructor
	 */
	limits_alarm_impl();
	/**
	 * Destructor
	 */
	virtual ~limits_alarm_impl();
	//! \}

	/** \name implementation of abstract interface 'FB'
  //! \{
	/**
	 * Perfoms the algorithm of the function block
	 */
	virtual void 	en();
  //! \}
	
  /** \name implementation of abstract interface 'limits_alarm'
  //! \{
  /**
   * Sets the input values of the function block. 
   * Should be called before \ref en().
   * \param x - value of x (process value)
   * \param h - value of high limit
   * \param l - value of low limit
   * \param eps - value of eps 
   */ 
  virtual void  set_input(const float x, const float h, const float l, const float eps);
  /**
   * Gets the input values of the function block. 
   * Should be called before \ref en().
   * \param x - value of x (process value)
   * \param h - value of high limit
   * \param l - value of low limit
   * \param eps - value of eps 
   */ 
  virtual void  get_input(float& x, float& h, float& l, float& eps) const;
  /**
   * Delivers the result of the algorithm.
   * Should be called after \ref en().
   * return true - value exceeds the limit. 
   */
  virtual bool  get_result() const;	
  /**
   * Is true, if value exceeds the high limit.
   * return true - value exceeds the high limit.  
   */
  virtual bool  is_high_alert() const;	
  /**
   * Is true, if value exceeds the low limit.
   * return true - value exceeds the low limit.  
   */
  virtual bool  is_low_alert() const;	
  /**
   * Sets the interface pointer to an instance for lower checker.
   * \param hyst - interface pointer to a hysteresis implementation
   */ 
  virtual void  set_lower_checker(hysteresis* hyst);
  /**
   * Sets the interface pointer to an instance for upper checker.
   * \param hyst - interface pointer to a hysteresis implementation
   */ 
  virtual void  set_upper_checker(hysteresis* hyst);

  //! \}
private:
  float	m_h; /**< higher bound */
  float m_l; /**< lower boun */ 
	float	m_x; /**< process value */
	float	m_eps; /**< confidential interval */

	bool	m_q; /**< output; true - value is inside the confidence interval */
  bool	m_qh; /**< output; true - value is beyound the higher bound */
  bool	m_ql; /**< output; true - value is beyound the lower bound */
  float m_eps_2; /**< half of the confidence interval */

  hysteresis* m_lower; /**< check, if pv falls below low limit */
  hysteresis* m_higher; /**< check, if pv exceeds high limit */ 
}; // limits_alarm_impl

#endif  //__LIMITS_ALARM_IMPL_H
